@@ -31,17 +31,16 @@ fn simple() {
     Package::new("bar", "0.0.1").publish();
 
     p.cargo("build")
-        .with_stderr(&format!(
+        .with_stderr(
             "\
-[UPDATING] `{reg}` index
+[UPDATING] `dummy-registry` index
 [DOWNLOADING] crates ...
-[DOWNLOADED] bar v0.0.1 (registry `[ROOT][..]`)
+[DOWNLOADED] bar v0.0.1 (registry `dummy-registry`)
 [COMPILING] bar v0.0.1
 [COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]s
 ",
-            reg = registry_path().to_str().unwrap()
-        ))
+        )
         .run();
 
     p.cargo("clean").run();
@@ -80,19 +79,18 @@ fn deps() {
     Package::new("bar", "0.0.1").dep("baz", "*").publish();
 
     p.cargo("build")
-        .with_stderr(&format!(
+        .with_stderr(
             "\
-[UPDATING] `{reg}` index
+[UPDATING] `dummy-registry` index
 [DOWNLOADING] crates ...
-[DOWNLOADED] [..] v0.0.1 (registry `[ROOT][..]`)
-[DOWNLOADED] [..] v0.0.1 (registry `[ROOT][..]`)
+[DOWNLOADED] [..] v0.0.1 (registry `dummy-registry`)
+[DOWNLOADED] [..] v0.0.1 (registry `dummy-registry`)
 [COMPILING] baz v0.0.1
 [COMPILING] bar v0.0.1
 [COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]s
 ",
-            reg = registry_path().to_str().unwrap()
-        ))
+        )
         .run();
 }
 
@@ -155,9 +153,10 @@ fn wrong_case() {
         .with_stderr(
             "\
 [UPDATING] [..] index
-error: no matching package named `Init` found
+error: no matching package found
+searched package name: `Init`
+perhaps you meant:      init
 location searched: registry [..]
-perhaps you meant: init
 required by package `foo v0.0.1 ([..])`
 ",
         )
@@ -190,9 +189,10 @@ fn mis_hyphenated() {
         .with_stderr(
             "\
 [UPDATING] [..] index
-error: no matching package named `mis_hyphenated` found
+error: no matching package found
+searched package name: `mis_hyphenated`
+perhaps you meant:      mis-hyphenated
 location searched: registry [..]
-perhaps you meant: mis-hyphenated
 required by package `foo v0.0.1 ([..])`
 ",
         )
@@ -277,10 +277,10 @@ fn bad_cksum() {
 [UPDATING] [..] index
 [DOWNLOADING] crates ...
 [DOWNLOADED] bad-cksum [..]
-[ERROR] failed to download replaced source registry `https://[..]`
+[ERROR] failed to download replaced source registry `crates-io`
 
 Caused by:
-  failed to verify the checksum of `bad-cksum v0.0.1 (registry `[ROOT][..]`)`
+  failed to verify the checksum of `bad-cksum v0.0.1 (registry `dummy-registry`)`
 ",
         )
         .run();
@@ -320,17 +320,16 @@ required by package `foo v0.0.1 ([..])`
     Package::new("notyet", "0.0.1").publish();
 
     p.cargo("build")
-        .with_stderr(format!(
+        .with_stderr(
             "\
-[UPDATING] `{reg}` index
+[UPDATING] `dummy-registry` index
 [DOWNLOADING] crates ...
-[DOWNLOADED] notyet v0.0.1 (registry `[ROOT][..]`)
+[DOWNLOADED] notyet v0.0.1 (registry `dummy-registry`)
 [COMPILING] notyet v0.0.1
 [COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]s
 ",
-            reg = registry_path().to_str().unwrap()
-        ))
+        )
         .run();
 }
 
@@ -370,7 +369,7 @@ fn package_with_path_deps() {
 
 Caused by:
   no matching package named `notyet` found
-  location searched: registry `https://github.com/rust-lang/crates.io-index`
+  location searched: registry `crates-io`
   required by package `foo v0.0.1 [..]`
 ",
         )
@@ -385,7 +384,7 @@ Caused by:
 [UPDATING] `[..]` index
 [VERIFYING] foo v0.0.1 ([CWD])
 [DOWNLOADING] crates ...
-[DOWNLOADED] notyet v0.0.1 (registry `[ROOT][..]`)
+[DOWNLOADED] notyet v0.0.1 (registry `dummy-registry`)
 [COMPILING] notyet v0.0.1
 [COMPILING] foo v0.0.1 ([CWD][..])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]s
@@ -419,7 +418,7 @@ fn lockfile_locks() {
             "\
 [UPDATING] `[..]` index
 [DOWNLOADING] crates ...
-[DOWNLOADED] bar v0.0.1 (registry `[ROOT][..]`)
+[DOWNLOADED] bar v0.0.1 (registry `dummy-registry`)
 [COMPILING] bar v0.0.1
 [COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]s
@@ -459,8 +458,8 @@ fn lockfile_locks_transitively() {
             "\
 [UPDATING] `[..]` index
 [DOWNLOADING] crates ...
-[DOWNLOADED] [..] v0.0.1 (registry `[ROOT][..]`)
-[DOWNLOADED] [..] v0.0.1 (registry `[ROOT][..]`)
+[DOWNLOADED] [..] v0.0.1 (registry `dummy-registry`)
+[DOWNLOADED] [..] v0.0.1 (registry `dummy-registry`)
 [COMPILING] baz v0.0.1
 [COMPILING] bar v0.0.1
 [COMPILING] foo v0.0.1 ([CWD])
@@ -507,8 +506,8 @@ fn yanks_are_not_used() {
             "\
 [UPDATING] `[..]` index
 [DOWNLOADING] crates ...
-[DOWNLOADED] [..] v0.0.1 (registry `[ROOT][..]`)
-[DOWNLOADED] [..] v0.0.1 (registry `[ROOT][..]`)
+[DOWNLOADED] [..] v0.0.1 (registry `dummy-registry`)
+[DOWNLOADED] [..] v0.0.1 (registry `dummy-registry`)
 [COMPILING] baz v0.0.1
 [COMPILING] bar v0.0.1
 [COMPILING] foo v0.0.1 ([CWD])
@@ -720,7 +719,7 @@ fn update_with_lockfile_if_packages_missing() {
             "\
 [UPDATING] `[..]` index
 [DOWNLOADING] crates ...
-[DOWNLOADED] bar v0.0.1 (registry `[ROOT][..]`)
+[DOWNLOADED] bar v0.0.1 (registry `dummy-registry`)
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]s
 ",
         )
@@ -767,7 +766,7 @@ fn update_lockfile() {
         .with_stderr(
             "\
 [DOWNLOADING] crates ...
-[DOWNLOADED] [..] v0.0.2 (registry `[ROOT][..]`)
+[DOWNLOADED] [..] v0.0.2 (registry `dummy-registry`)
 [COMPILING] bar v0.0.2
 [COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]s
@@ -790,7 +789,7 @@ fn update_lockfile() {
         .with_stderr(
             "\
 [DOWNLOADING] crates ...
-[DOWNLOADED] [..] v0.0.3 (registry `[ROOT][..]`)
+[DOWNLOADED] [..] v0.0.3 (registry `dummy-registry`)
 [COMPILING] bar v0.0.3
 [COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]s
@@ -850,7 +849,7 @@ fn dev_dependency_not_used() {
             "\
 [UPDATING] `[..]` index
 [DOWNLOADING] crates ...
-[DOWNLOADED] [..] v0.0.1 (registry `[ROOT][..]`)
+[DOWNLOADED] [..] v0.0.1 (registry `dummy-registry`)
 [COMPILING] bar v0.0.1
 [COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]s
@@ -946,7 +945,7 @@ fn updating_a_dep() {
             "\
 [UPDATING] `[..]` index
 [DOWNLOADING] crates ...
-[DOWNLOADED] bar v0.0.1 (registry `[ROOT][..]`)
+[DOWNLOADED] bar v0.0.1 (registry `dummy-registry`)
 [COMPILING] bar v0.0.1
 [COMPILING] a v0.0.1 ([CWD]/a)
 [COMPILING] foo v0.0.1 ([CWD])
@@ -975,7 +974,7 @@ fn updating_a_dep() {
             "\
 [UPDATING] `[..]` index
 [DOWNLOADING] crates ...
-[DOWNLOADED] bar v0.1.0 (registry `[ROOT][..]`)
+[DOWNLOADED] bar v0.1.0 (registry `dummy-registry`)
 [COMPILING] bar v0.1.0
 [COMPILING] a v0.0.1 ([CWD]/a)
 [COMPILING] foo v0.0.1 ([CWD])
@@ -1033,7 +1032,7 @@ fn git_and_registry_dep() {
 [UPDATING] [..]
 [UPDATING] [..]
 [DOWNLOADING] crates ...
-[DOWNLOADED] a v0.0.1 (registry `[ROOT][..]`)
+[DOWNLOADED] a v0.0.1 (registry `dummy-registry`)
 [COMPILING] a v0.0.1
 [COMPILING] b v0.0.1 ([..])
 [COMPILING] foo v0.0.1 ([CWD])
@@ -1110,7 +1109,7 @@ fn update_publish_then_update() {
             "\
 [UPDATING] [..]
 [DOWNLOADING] crates ...
-[DOWNLOADED] a v0.1.1 (registry `[ROOT][..]`)
+[DOWNLOADED] a v0.1.1 (registry `dummy-registry`)
 [COMPILING] a v0.1.1
 [COMPILING] foo v0.5.0 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]s
@@ -1188,7 +1187,7 @@ fn update_transitive_dependency() {
         .with_stderr(
             "\
 [DOWNLOADING] crates ...
-[DOWNLOADED] b v0.1.1 (registry `[ROOT][..]`)
+[DOWNLOADED] b v0.1.1 (registry `dummy-registry`)
 [COMPILING] b v0.1.1
 [COMPILING] a v0.1.0
 [COMPILING] foo v0.5.0 ([..])
@@ -1297,9 +1296,9 @@ fn update_multiple_packages() {
         .run();
 
     p.cargo("build")
-        .with_stderr_contains("[DOWNLOADED] a v0.1.1 (registry `[ROOT][..]`)")
-        .with_stderr_contains("[DOWNLOADED] b v0.1.1 (registry `[ROOT][..]`)")
-        .with_stderr_contains("[DOWNLOADED] c v0.1.1 (registry `[ROOT][..]`)")
+        .with_stderr_contains("[DOWNLOADED] a v0.1.1 (registry `dummy-registry`)")
+        .with_stderr_contains("[DOWNLOADED] b v0.1.1 (registry `dummy-registry`)")
+        .with_stderr_contains("[DOWNLOADED] c v0.1.1 (registry `dummy-registry`)")
         .with_stderr_contains("[COMPILING] a v0.1.1")
         .with_stderr_contains("[COMPILING] b v0.1.1")
         .with_stderr_contains("[COMPILING] c v0.1.1")
@@ -1438,10 +1437,11 @@ fn use_semver_package_incorrectly() {
         .with_status(101)
         .with_stderr(
             "\
-error: no matching package named `a` found
-location searched: [..]
+error: no matching package found
+searched package name: `a`
 prerelease package needs to be specified explicitly
 a = { version = \"0.1.1-alpha.0\" }
+location searched: [..]
 required by package `b v0.1.0 ([..])`
 ",
         )
@@ -1695,119 +1695,6 @@ fn bump_version_dont_update_registry() {
         .run();
 }
 
-#[cargo_test]
-fn old_version_req() {
-    let p = project()
-        .file(
-            "Cargo.toml",
-            r#"
-                [project]
-                name = "bar"
-                version = "0.5.0"
-                authors = []
-
-                [dependencies]
-                remote = "0.2*"
-            "#,
-        )
-        .file("src/main.rs", "fn main() {}")
-        .build();
-
-    Package::new("remote", "0.2.0").publish();
-
-    p.cargo("build")
-        .with_stderr(
-            "\
-warning: parsed version requirement `0.2*` is no longer valid
-
-Previous versions of Cargo accepted this malformed requirement,
-but it is being deprecated. This was found when parsing the manifest
-of bar 0.5.0, and the correct version requirement is `0.2.*`.
-
-This will soon become a hard error, so it's either recommended to
-update to a fixed version or contact the upstream maintainer about
-this warning.
-
-warning: parsed version requirement `0.2*` is no longer valid
-
-Previous versions of Cargo accepted this malformed requirement,
-but it is being deprecated. This was found when parsing the manifest
-of bar 0.5.0, and the correct version requirement is `0.2.*`.
-
-This will soon become a hard error, so it's either recommended to
-update to a fixed version or contact the upstream maintainer about
-this warning.
-
-[UPDATING] [..]
-[DOWNLOADING] crates ...
-[DOWNLOADED] [..]
-[COMPILING] [..]
-[COMPILING] [..]
-[FINISHED] [..]
-",
-        )
-        .run();
-}
-
-#[cargo_test]
-fn old_version_req_upstream() {
-    let p = project()
-        .file(
-            "Cargo.toml",
-            r#"
-                [project]
-                name = "bar"
-                version = "0.5.0"
-                authors = []
-
-                [dependencies]
-                remote = "0.3"
-            "#,
-        )
-        .file("src/main.rs", "fn main() {}")
-        .build();
-
-    Package::new("remote", "0.3.0")
-        .file(
-            "Cargo.toml",
-            r#"
-                [project]
-                name = "remote"
-                version = "0.3.0"
-                authors = []
-
-                [dependencies]
-                bar = "0.2*"
-            "#,
-        )
-        .file("src/lib.rs", "")
-        .publish();
-    Package::new("bar", "0.2.0").publish();
-
-    p.cargo("build")
-        .with_stderr(
-            "\
-[UPDATING] [..]
-[DOWNLOADING] crates ...
-[DOWNLOADED] [..]
-warning: parsed version requirement `0.2*` is no longer valid
-
-Previous versions of Cargo accepted this malformed requirement,
-but it is being deprecated. This was found when parsing the manifest
-of remote 0.3.0, and the correct version requirement is `0.2.*`.
-
-This will soon become a hard error, so it's either recommended to
-update to a fixed version or contact the upstream maintainer about
-this warning.
-
-[COMPILING] [..]
-[COMPILING] [..]
-[FINISHED] [..]
-",
-        )
-        .run();
-}
-
 #[cargo_test]
 fn toml_lies_but_index_is_truth() {
     Package::new("foo", "0.2.0").publish();
